/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_SceneGraphTypes.h                            //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLSCENEGRAPHTYPES_H
#define GLSCENEGRAPHTYPES_H

#include "X3DTypes.h"

#include <GL/gl.h>
#include <list>
#include <vector>

namespace X3DTK {
namespace GL {

class X3DNode;
/// SFNode class 
typedef X3DNode *SFNode;
/// MFNode.
typedef std::list<SFNode> MFNode;

/// Defines a structure equivalent to the GL_C4F_N3F_V3F used in vertex arrays.
typedef struct
{
  SFColorRGBA color;
  SFVec3f normal;
  SFVec3f vertex;
} C4F_N3F_V3F;

/// Defines a structure equivalent to the GL_N3F_V3F used in vertex arrays.
typedef struct 
{
  SFVec3f normal;
  SFVec3f vertex;
} N3F_V3F;

/// Defines a structure equivalent to the GL_T2F_C4F_N3F_V3F used in vertex arrays.  
typedef struct
{
  SFVec2f texCoord;
  SFColorRGBA color;
  SFVec3f normal;
  SFVec3f vertex;
} T2F_C4F_N3F_V3F;

/// Defines a structure equivalent to the GL_T2F_N3F_V3F used in vertex arrays.
typedef struct 
{
  SFVec2f texCoord;
  SFVec3f normal;
  SFVec3f vertex;
} T2F_N3F_V3F;

/// Defines a structure equivalent to the GL_C4UB_V3F used in vertex arrays.
typedef struct
{
  unsigned char r, g, b, a;
  SFVec3f vertex;
} C4UB_V3F;
}
}

#endif
