/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MESH_MeshVerticesDrawerStateVariables.h         //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESH_VERTICESDRAWERSTATEVARIABLES_H
#define MESH_VERTICESDRAWERSTATEVARIABLES_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"

#include <list>
#include <vector>

namespace X3DTK {
namespace MESH {

// State variables for the Drawer processor.

class X3DTK_API MeshVerticesDrawerStateVariables : public StateVariables
{
public:
  MeshVerticesDrawerStateVariables();

  // Initializes and finishes the traversal.
  void init();
  void finish();

  // Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);  
  
  // Pops the transformation matrix.
  void popMatrix();

  // Saves current matrix state and returns index in saved vector.
  // Call storeMesh() before calling ths function
  int getCurrentMeshId();
  // Saves mesh and associated current matrix.
  void storeMesh(Mesh *mesh);
  
  // Get i Mesh of the model.
  inline Mesh *getMesh(int i) const {return _meshVector[i];};
  // Gets stored transformation matrix associated with a given Mesh.
  inline const SFMatrix34f &getMatrix(int i) const {return _matrixVector[i];};

  // Whether or not primitive IDs are pushed.
  inline void setSelectionMode(bool selectionMode) {_selectionMode = selectionMode;};
  inline bool getSelectionMode() const {return _selectionMode;};
    
  // Whether each mesh is displayed with a specific color
  void changeOneColorPerMesh() {_oneColorPerMesh = !_oneColorPerMesh;};
  inline bool getOneColorPerMesh() const {return _oneColorPerMesh;};
  
private: 
  std::list<SFMatrix34f> _matrixStack;
  std::vector<SFMatrix34f> _matrixVector;
  std::vector<Mesh *> _meshVector;
  bool _selectionMode;
  bool _oneColorPerMesh;
};

}
}

#endif
