/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3D_ColorInterpolator.h                         //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef COLORINTERPOLATOR
#define COLORINTERPOLATOR

#include "X3DTypes.h"
#include "X3D_X3DInterpolatorNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/interpolator.html">
/// Interpolator component</a> defining an ColorInterpolator
/// (<a href="../../doc/X3DSpecs/Part01/components/interpolator.html#ColorInterpolator">
/// documentation</a>).

class X3DTK_API ColorInterpolator : public X3DInterpolatorNode
{
public:
  /// Constructor.
  ColorInterpolator();
  /// Constructs a ColorInterpolator from its attributes.
  ColorInterpolator(const MFFloat &key,
                    const MFColor &keyValue);
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Virtual destructor.
  virtual ~ColorInterpolator();

  /// Sets the keyValue attribute.
  void setKeyValue(const MFColor &keyValue);

  /// Gets the keyValue attribute.
  inline const MFColor &getKeyValue() const {return _keyValue;};

  /// Loads the attributes from a X3DFileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString &write(SFString &output) const;

protected:
  /// Copy constructor.
  ColorInterpolator(const ColorInterpolator &N);

private:
  /// Values of the piecewise-linear function corresponding to the key attribute.
  MFColor _keyValue;
};

}
}

#endif
