/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3D_WorldCoordTranslatorStateVariables.h        //
////////////////////////////////////////////////////////////////////////////////

#ifndef WORLDCOORDTRANSLATORGLOBALVARIABLES_H
#define WORLDCOORDTRANSLATORGLOBALVARIABLES_H

#include "X3D_SceneGraphTypes.h"
#include "StateVariables.h"

#include <list>

namespace X3DTK {
namespace X3D {

/// StateVariables for the WorldCoordTranslator module.

class X3DTK_API WorldCoordTranslatorStateVariables : public StateVariables
{
public:
  /// Constructor.
  WorldCoordTranslatorStateVariables();
  /// Destructor.
  virtual ~WorldCoordTranslatorStateVariables();

  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();

  /// Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);  
  
  /// Pops the transformation matrix.
  void popMatrix();
  
  /// Gets the current transformation matrix.
  SFMatrix34f getMatrix() const {return _matrixStack.front();};

private: 
  std::list<SFMatrix34f> _matrixStack;
};

}
}

#endif
