/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_Box.h                                        //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLBOX_H
#define GLBOX_H

#include "GL_X3DGeometry3DNode.h"

namespace X3DTK {
namespace GL {

class BoxDrawArray;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode Box.

class X3DTK_API Box : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Box();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Box();

  /// Sets the size attribute.
  void setSize(const SFVec3f &size);
  /// Sets the box array.
  void setBoxArray(BoxDrawArray *boxArray);

  /// Gets the size attribute.
  inline const SFVec3f &getSize() const {return _size;};
  /// Gets the box array.
  inline BoxDrawArray *getBoxArray() const {return _boxArray;};
 
  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void draw() const;  
  
protected:     
  /// Copy constructor.
  Box(const Box &B);

private:  
  /// Size attribute.
  SFVec3f _size;
  /// Box array.
  BoxDrawArray *_boxArray;
};

}
}

#endif
