/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_Sphere.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLSPHERE_H
#define GLSPHERE_H

#include "GL_X3DGeometry3DNode.h"

namespace X3DTK {
namespace GL {

class SphereDrawArray;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Sphere.

class X3DTK_API Sphere : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Sphere();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Sphere();
  
  /// Sets the radius attribute.
  void setRadius(const SFFloat &radius);
  /// Sets the sphere array.
  void setSphereDrawArray(SphereDrawArray *sphereArray);
  
  /// Gets the radius attribute.
  inline const SFFloat &getRadius() const {return _radius;};
  /// Gets the sphere array.
  inline SphereDrawArray *getSphereArray() const {return _sphereArray;};
  
  /// Updates the attributes of the node.
  virtual void update();
  /// Draws the node in an openGL context.
  virtual void draw() const; 
  
protected:
  /// Copy constructor.	 
  Sphere(const Sphere &S);

private:  
  /// Radius attribute.
  SFFloat _radius;
  /// Sphere array.
  SphereDrawArray *_sphereArray;
};

}
}

#endif
