/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFTemplateEdge.h                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEEDGE_H
#define TEMPLATEEDGE_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_SFTemplateEdgeContent.h"

namespace X3DTK {
namespace MESH {

template<class A, class B, class C> class SFTemplateVertex;
template<class A, class B, class C> class SFTemplateFace;

/// Class providing a template edge for the Mesh.

template<class EData, class FData, class VData>
class SFTemplateEdge 
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
  template<class A, class B, class C> friend class SFTemplateDirectEdge;
  template<class A, class B, class C> friend class SFTemplateIndirectEdge;
public:
  /// Returns the symetric edge.
  inline SFTemplateEdge *getSymetric() {return _symetric;};
  /// Returns the Vertex from.
  inline SFTemplateVertex<VData, EData, FData> *getFromVertex() const {return (_direct ? _edgeContent->getFromVertex() : _edgeContent->getToVertex());};
  /// Gets the Vertex to.
  inline SFTemplateVertex<VData, EData, FData> *getToVertex() const {return (_direct ? _edgeContent->getToVertex() : _edgeContent->getFromVertex());};
  /// Gets the left faces.
  inline const MFTemplateFace<FData, VData, EData> &getLeftFaces() const {return (_direct ? _edgeContent->get1Faces() : _edgeContent->get2Faces());};
  /// Gets the right faces.
  inline const MFTemplateFace<FData, VData, EData> &getRightFaces() const {return (_direct ? _edgeContent->get2Faces() : _edgeContent->get1Faces());};
  /// Gets the faces.
  MFTemplateFace<FData, VData, EData> getFaces() const 
  {
    MFTemplateFace<FData, VData, EData> faces;
    const MFTemplateFace<FData, VData, EData> &r1faces = _edgeContent->get1Faces();
    const MFTemplateFace<FData, VData, EData> &r2faces = _edgeContent->get2Faces();
    
    //faces.reserve(r1faces.size() + r2faces.size() + 1);
    for (typename MFTemplateFace<FData, VData, EData>::const_iterator it = r1faces.begin(); it != r1faces.end(); ++it)
      faces.push_back(*it);

    for (typename MFTemplateFace<FData, VData, EData>::const_iterator it = r2faces.begin(); it != r2faces.end(); ++it)
      faces.push_back(*it);

    return faces;
  };
  /// Returns true if is a boundary.
  inline bool isBoundary() const {return _edgeContent->isBoundary();};
  /// Gets the data.
  inline EData &data() {return _edgeContent->data();};
  /// Gets the data.
  inline const EData &data() const {return _edgeContent->data();};

protected:  
  void setDirect(bool value) {_direct = value;};

private:
  SFTemplateEdgeContent<EData, FData, VData> *const _edgeContent;
  bool _direct;
  SFTemplateEdge *_symetric;
  
  /// Constructor.
  SFTemplateEdge(SFTemplateEdgeContent<EData, FData, VData> *const edgeContent)
  : _edgeContent(edgeContent), _symetric(0)
  {};

  /// Destructor.
  ~SFTemplateEdge() {};
  
  void setSymetric(SFTemplateEdge *symetric) {_symetric = symetric;};
  
  /// Adds a left face.
  void addLeftFace(SFTemplateFace<FData, VData, EData> *face) 
  {
    if (_direct)
      _edgeContent->add1Face(face);
    else
      _edgeContent->add2Face(face);    
  };
  /// Adds a right face.
  void addRightFace(SFTemplateFace<FData, VData, EData> *face) 
  {
    if (_direct)
      _edgeContent->add2Face(face);
    else
      _edgeContent->add1Face(face);    
  };
};


}
}

#endif
