/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFTemplateFace.h                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEFACE_H
#define TEMPLATEFACE_H

#include "MESH_SceneGraphTypes.h"

#include <vector>
#include <iostream>

namespace X3DTK {
namespace MESH {

template<class A, class B, class C> class SFTemplateEdge;

/// Template class providing a Face for the Mesh structure.

template<class FData, class VData, class EData>
class SFTemplateFace 
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
public:
  /// Gets the index.
  inline unsigned int getIndex() const {return _index;};
  /// Gets the edges of the faces. The order is given b the order of definition of the face.
  inline const MFTemplateEdge<EData, FData, VData> &getEdges() const {return _edges;};
  /// Gets the data.
  inline FData &data() {return _data;};
  /// Gets the data.
  inline const FData &data() const {return _data;};

private:
  const unsigned int _index;
  MFTemplateEdge<EData, FData, VData> _edges;
  FData _data;
  
  /// Constructor.
  SFTemplateFace(unsigned int i)
  : _index(i)
  {
  };
  
  /// Destructor.
  ~SFTemplateFace() {};
  
  /// Adds an edge.
  void addEdge(SFTemplateEdge<EData, FData, VData> *edge)
  {
    _edges.push_back(edge);
  };
};

}
}

#endif
