/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TemplateVertexSet.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEVERTEXSET_H
#define TEMPLATEVERTEXSET_H

#include "MESH_X3DNode.h"
#include "MESH_SFTemplateVertex.h"

namespace X3DTK {
namespace MESH {

template<class VData, class EData, class FData>
class TemplateVertexSet : public X3DNode
{
public:
  /// Constructor.
  TemplateVertexSet()
  : X3DNode() 
  {
  };
  
  /// Clones the node.
  virtual SFNode clone() const
  {
    return new TemplateVertexSet(*this);  
  };
  
  /// Virtual destructor.
  virtual ~TemplateVertexSet() 
  {
    // delete vertices
    for (typename MFTemplateVertex<VData, EData, FData>::iterator it = _vertices.begin(); it != _vertices.end(); ++it)
      delete (*it);
  };
    
  /// Creates a new vertex.
  SFTemplateVertex<VData, EData, FData> *createVertex() 
  {
    SFTemplateVertex<VData, EData, FData> *V = new SFTemplateVertex<VData, EData, FData>(_vertices.size());
    _vertices.push_back(V);
    return V;
  };
  /// Creates a new vertex at the position i.
  SFTemplateVertex<VData, EData, FData> *createVertex(unsigned int i) 
  {
    if (i > _vertices.size())
      _vertices.resize(i + 1, 0);
    else
    {
      if (_vertices[i] != 0)  
      {
        cx3d << "TemplateVertexSet::createVertex: cannot create a Vertex at index " << i << ", a Vertex already exists!" << std::endl;
        return 0;
      }
    }
    
    SFTemplateVertex<VData, EData, FData> *V = new SFTemplateVertex<VData, EData, FData>(i);  
    _vertices[i] = V;
    
    return V;
  };  
  
  /// Gets the vertices.
  inline const MFTemplateVertex<VData, EData, FData> &getVertices() const {return _vertices;};

protected:
  /// Copy constructor.
  TemplateVertexSet(const TemplateVertexSet &S)
  : X3DNode(S), _vertices(S._vertices)
  {
  };   

private:
  MFTemplateVertex<VData, EData, FData> _vertices; 
};  

}
}

#endif
