#include "MESH_Mesh.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_Coordinate.h"
#include "X3D_Normal.h"
#include "X3D_TextureCoordinate.h"
#include "X3D_Color.h"
#include "X3D_ColorRGBA.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

VertexData::VertexData()
: _point(SFPoint3f(0.0f, 0.0f, 0.0f))
{
}

VertexData::~VertexData()
{
}

void VertexData::setPoint(const SFPoint3f &point)
{
  _point = point;
}

void VertexData::setNormalOfFace(const SFVec3f &normal, SFTemplateFace<FaceData, VertexData, EdgeData> *face)
{
  // iterating the normals to find whether another one is identical.
  unsigned short i = 0;
  for (MFVec3f::iterator it = _normalArray.begin(); it != _normalArray.end(); ++it)
  {
    // found an identical normal.
    if (*it == normal)
    {
      _normalMap[face] = i;
      return;
    }
    ++i;
  }  
  // did not found an identical normal.
  _normalMap[face] = _normalArray.size();
  _normalArray.push_back(normal);  
}

void VertexData::setColorOfFace(const SFColorRGBA &color, SFTemplateFace<FaceData, VertexData, EdgeData> *face)
{
  unsigned short i = 0;
  for (MFColorRGBA::iterator it = _colorArray.begin(); it != _colorArray.end(); ++it)
  {
    if (*it == color)
    {
      _colorMap[face] = i;
      return;
    }
    ++i;
  }  
  _colorMap[face] = _colorArray.size();
  _colorArray.push_back(color);  
}

void VertexData::setTexCoordOfFace(const SFVec2f &texCoord, SFTemplateFace<FaceData, VertexData, EdgeData> *face)
{
  unsigned short i = 0;
  for (MFVec2f::iterator it = _texCoordArray.begin(); it != _texCoordArray.end(); ++it)
  {
    if (*it == texCoord)
    {
      _texCoordMap[face] = i;
      return;
    }
    ++i;
  }  
  _texCoordMap[face] = _texCoordArray.size();
  _texCoordArray.push_back(texCoord);  
}

const SFVec3f &VertexData::getNormalOfFace(SFTemplateFace<FaceData, VertexData, EdgeData> *face) const 
{
  map<SFTemplateFace<FaceData, VertexData, EdgeData> *, unsigned short>::const_iterator it = _normalMap.find(face);
  if (it != _normalMap.end())
    return _normalArray[(*it).second];
    
  return SFVec3f::null;  
};

const SFColorRGBA &VertexData::getColorOfFace(SFTemplateFace<FaceData, VertexData, EdgeData> *face) const
{
  map<SFTemplateFace<FaceData, VertexData, EdgeData> *, unsigned short>::const_iterator it = _colorMap.find(face);
  if (it != _colorMap.end())
    return _colorArray[(*it).second];
    
  return SFColorRGBA::null;  
}

const SFVec2f &VertexData::getTexCoordOfFace(SFTemplateFace<FaceData, VertexData, EdgeData> *face) const
{
  map<SFTemplateFace<FaceData, VertexData, EdgeData> *, unsigned short>::const_iterator it = _texCoordMap.find(face);
  if (it != _texCoordMap.end())
    return _texCoordArray[(*it).second];
    
  return SFVec2f::null;  
};

EdgeData::EdgeData()
{
}

FaceData::FaceData()
{
}

void FaceData::setNormal(const SFVec3f &normal)
{
  _normal = normal;
}

void FaceData::setColor(const SFColorRGBA &color)
{
  _color = color;
}

MeshData::MeshData()
{
}

void MeshData::setNormal(bool value)
{
  _normal = value;
}

void MeshData::setColor(bool value)
{
  _color = value;
}

void MeshData::setTexCoord(bool value)
{
  _texCoord = value;
}

void MeshData::setNormalPerVertex(bool value)
{
  _normalPerVertex = value;
}

void MeshData::setColorPerVertex(bool value)
{
  _colorPerVertex = value;
}

}
}
