/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3D_X3DComponentCreator.h                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENTCREATOR_H
#define X3DCOMPONENTCREATOR_H

#include "X3DComponent.h"
#include "X3D_CreationFunction.h"

namespace X3DTK {
namespace X3D {

/// Abstract class defining the concept of component creator.

class X3DTK_API X3DComponentCreator : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentCreator();
  /// Destructor.
  virtual ~X3DComponentCreator() = 0;
  
  /// Gets the creation of name.
  CreationFunction *getCreationFunctionOf(const SFString &name) const;
  /// Returns the list of functions
  inline CreationDict getCreationDict() const {return _creationDict;};
  
  /// Returns TRUE if the component contains the type name.
  bool contains(const SFString &name) const;
  
protected: 
  template<class T>
  void defineNode()
  {
    defineCreationFunction<T, X3DComponentCreator>(&X3DComponentCreator::create<T>);
  };

private:
  template<class T>
  SFNode create() const
  {
    return new T();
  };
  
  /// Define a new creation function.
  template<class T, class C> void defineCreationFunction(SFNode (C::*ptrF)() const)
  {
    SFType::beginFunctionDefinition();
    SFNode t = new T();
    SFString name = t->getTypeName();
    SFComponent *TComponent = t->getType()->getComponent();
    //checking if the node belongs to the component.
    if (TComponent != component)
    {
      SFString sg1, sg2;
      if (component->getSceneGraphName() != "")
        sg1 = " of the " + component->getSceneGraphName() + " scene graph";
      if (t->getType()->getSceneGraphName() != "")
        sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

      cx3d << "warning: defineNode for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
      cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;    
    }         
    else
    {
      CreationFunction *CF = new CreationFunction(reinterpret_cast<ptrToCreationFunction>(ptrF), this);
      _creationDict.insert(std::pair<SFString, CreationFunction *>(name, CF));
    }
    delete t;
    SFType::endFunctionDefinition();
  };

  CreationDict _creationDict;
};

}
}

#endif
