/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                             MESH_MeshFacesDrawer.h                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESH_FACESDRAWER_H
#define MESH_FACESDRAWER_H

#include "MESH_MeshFacesDrawerStateVariables.h"

#include "X3DOnePassProcessor.h"
#include "MESH_SceneGraphTypes.h"

namespace X3DTK {
namespace MESH {

// Processor drawing the mesh from the Mesh scene graph.

class X3DTK_API MeshFacesDrawer : public X3DOnePassProcessor
{
public:
  MeshFacesDrawer();
  virtual ~MeshFacesDrawer();
  
  void changeOneColorPerMesh();
  
  void draw(X3DNode *N, bool selection = false);
  
  // Get ith Mesh of the scene.
  inline Mesh *getMesh(int i) const {return stateVariables->getMesh(i);};
  // Gets stored transformation matrix associated with a given Mesh.
  inline const SFMatrix34f &getMatrix(int i) const {return stateVariables->getMatrix(i);};
  /// Gets the ith face of the jth Mesh.
  inline Mesh::SFFace *getFaceOfMesh(int i, int j) const {return (stateVariables->getMesh(j)->getFaces())[i];};
  
protected:
  MeshFacesDrawerStateVariables* stateVariables;
  
/*  // Draw methods.
  void drawVertex(const Mesh::SFVertex *vertex);
  void drawEdge(Mesh::SFEdge *edge);*/
};

}
}

#endif

