#include "MESH_MeshNormalUpdaterVerticesCoreVisitor.h"

using namespace std;

namespace X3DTK {
namespace MESH {

MeshNormalUpdaterVerticesCoreVisitor::MeshNormalUpdaterVerticesCoreVisitor()
{
  defineEnterFunction(&MeshNormalUpdaterVerticesCoreVisitor::enterMesh);
}

void MeshNormalUpdaterVerticesCoreVisitor::enterMesh(Mesh *M) const
{
  const Mesh::MFVertex &vertices = M->getVertices();
  for (Mesh::MFVertex::const_iterator v = vertices.begin(), end = vertices.end() ; v != end; ++v)
  {
    SFVec3f normal;

    // Compute point normal as an area weighted average of faces' normals
    const Mesh::MFFace &faces = (*v)->getFaces();

    for (Mesh::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
      normal = normal + (*f)->data().getNormal();

    normal.normalize();

    // Store computed normal in VertexFData
    for (Mesh::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
    {  
      if ((*v)->data().getNormalOfFace(*f).norm() < 1E-4)
        (*v)->data().setNormalOfFace(normal, *f);
    } 
  }
}

} 
} 
