#include "MESH_MeshVerticesDrawerCoreVisitor.h"
#include "MESH_Transform.h"

#include <GL/gl.h>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

MeshVerticesDrawerCoreVisitor::MeshVerticesDrawerCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&MeshVerticesDrawerCoreVisitor::enterTransform);
  defineEnterFunction(&MeshVerticesDrawerCoreVisitor::enterMesh);
  defineLeaveFunction(&MeshVerticesDrawerCoreVisitor::leaveTransform);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<MeshVerticesDrawerStateVariables>();
}

void MeshVerticesDrawerCoreVisitor::enterTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->pushMatrix(T->getTransform());
  glPushMatrix();
  glMultMatrixf(T->getTransform().toFloat16());
}

void MeshVerticesDrawerCoreVisitor::leaveTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->popMatrix();
  glPopMatrix();
}

void MeshVerticesDrawerCoreVisitor::enterMesh(Mesh *M) const
{
  stateVariables->storeMesh(M);
  int meshId = stateVariables->getCurrentMeshId();

  if (stateVariables->getSelectionMode())
    glPushName(meshId);

  glColor3f(1.0f, 0.9f, 0.8f);
  if (stateVariables->getSelectionMode())
    glPointSize(5.0f);
  else
    glPointSize(1.0f);
    
  glDisable(GL_LIGHTING);  
  
  const Mesh::MFVertex &vertices = M->getVertices();

  if (stateVariables->getSelectionMode())
  {
    for (Mesh::MFVertex::const_iterator v = vertices.begin(), end = vertices.end() ; v != end; ++v)
    {
      glPushName((*v)->getIndex());
      SFPoint3f point = (*v)->data().getPoint();
      glRasterPos3fv(point.f_data());
      glPopName();
    }
    glPopName();
  }
  else
  {
    glBegin(GL_POINTS);
    for (Mesh::MFVertex::const_iterator v = vertices.begin(), end = vertices.end() ; v != end; ++v)
    {
      SFPoint3f point = (*v)->data().getPoint();
      glVertex3fv(point.f_data());
    }
    glEnd();
  } 
}

}
}
