#include "MESH_MeshX3DBuilderStateVariables.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

MeshX3DBuilderStateVariables::MeshX3DBuilderStateVariables()
: StateVariables(), _root(0), _cloner(X3DProcessor::getInstanceOf<Cloner>())
{
}

MeshX3DBuilderStateVariables::~MeshX3DBuilderStateVariables()
{
}

void MeshX3DBuilderStateVariables::init()
{
  _root = 0;
  _nodeStack.push_front(0);
}

void MeshX3DBuilderStateVariables::finish()
{
  _nodeStack.clear();
  _nodeCoupleMap.clear();
}

void MeshX3DBuilderStateVariables::addCoupleNode(SFNode N, X3D::SFNode NN)
{
  _nodeCoupleMap[N] = NN;  
  
  SFString name = X3DAbstractNode::DEFDict.getNameOfNode(N);
  if (name != "")
    X3DAbstractNode::DEFDict.insert(name, NN);

  // init root node.
  if (_root == 0)
    _root = NN;
}

void MeshX3DBuilderStateVariables::pushNode(X3D::SFNode N)
{
  X3D::SFNode T = getTop();
  if (T != 0)
    T->addChild(N);
    
  // filling the stack.    
  _nodeStack.push_front(N);
}

void MeshX3DBuilderStateVariables::popNode()
{
  _nodeStack.pop_front();
}

X3D::SFNode MeshX3DBuilderStateVariables::getNode(SFNode N) const
{
  map<SFNode, X3D::SFNode>::const_iterator it = _nodeCoupleMap.find(N);
  if (it != _nodeCoupleMap.end())
    return (*it).second;
    
  return 0;  
}

X3D::X3DNode *MeshX3DBuilderStateVariables::clone(X3D::X3DNode *N)
{
  return static_cast<X3D::X3DNode *>(_cloner->clone(N));
}

}
}
