/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MESH_MeshX3DBuilderStateVariables.h                 //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHX3DBUILDERGLOBALVARIABLES_H
#define MESHX3DBUILDERGLOBALVARIABLES_H

#include "StateVariables.h"
#include "MESH_X3DNode.h"
#include "X3D_X3DNode.h"
#include "Cloner.h"

#include <map>

namespace X3DTK {
namespace MESH {

/// StateVariables for the MeshX3DBuilder module.

class X3DTK_API MeshX3DBuilderStateVariables : public StateVariables
{
public:
  /// Constructor.
  MeshX3DBuilderStateVariables();
  /// Destructor.
  virtual ~MeshX3DBuilderStateVariables();
  /// Initializes the traversal.
  virtual void init();
  /// Finishes the traversal.
  virtual void finish();
  
  /// Adds the couple of nodes.
  void addCoupleNode(SFNode N, X3D::SFNode NN);
  /// Pushes the node.
  void pushNode(X3D::SFNode N);
  /// Pops the node.
  void popNode();
    
  /// Gets the root.
  inline X3D::SFNode getRoot() const {return _root;};
  /// Gets the top of the stack.
  inline X3D::SFNode getTop() const {return (!_nodeStack.empty() ? _nodeStack.front() : 0);};
  /// Gets the X3D::SFNode related with N. Returns 0 if N not found.
  X3D::SFNode getNode(SFNode N) const;
  /// Clones the node.
  X3D::X3DNode *clone(X3D::X3DNode *N);
    
private:
  X3D::SFNode _root;
  X3D::MFNode _nodeStack;
  std::map<SFNode, X3D::SFNode> _nodeCoupleMap;
  Cloner *_cloner;
};

}
}

#endif
