#include "X3D_MeshBuilder.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_Coordinate.h"
#include "X3D_Normal.h"
#include "X3D_TextureCoordinate.h"
#include "X3D_Color.h"
#include "X3D_ColorRGBA.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

void TemplateMeshBuilderGeometryTraits<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, IndexedFaceSet>::init(MESH::TemplateMesh<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData> *mesh, IndexedFaceSet *I) 
{
  // normals
  mesh->data().setNormal(I->getNormal() != 0);
  mesh->data().setNormalPerVertex(I->getNormalPerVertex());
  
  // colors
  mesh->data().setColor(I->getColor() != 0);
  mesh->data().setColorPerVertex(I->getColorPerVertex());
  
  // texCoords
  mesh->data().setTexCoord(I->getTexCoord() != 0);
}

void TemplateMeshBuilderGeometryTraits<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, IndexedFaceSet>::processNewFace(MESH::TemplateMesh<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData> *mesh, IndexedFaceSet *I, MESH::SFTemplateFace<MESH::FaceData, MESH::VertexData, MESH::EdgeData> *F)
{
}

void TemplateMeshBuilderGeometryTraits<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, IndexedFaceSet>::finish(MESH::TemplateMesh<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData> *mesh, IndexedFaceSet *I) 
{
  MFVec3f pointArray;
  MFVec3f normalArray;
  MFColorRGBA colorArray;
  MFVec2f texCoordArray;
  MFInt32 coordIndex;
  MFInt32 normalIndex;
  MFInt32 colorIndex;
  MFInt32 texCoordIndex;

  X3D::Normal *NormalNode = dynamic_cast<X3D::Normal *>(I->getNormal());
  if (NormalNode != 0)
    normalArray = NormalNode->getVector();
  
  if (I->getColor() != 0)
  {
    if (I->getColor()->getTypeName() == "ColorRGBA")
      colorArray = dynamic_cast<X3D::ColorRGBA *>(I->getColor())->getColor();
    else  
      colorArray = dynamic_cast<X3D::Color *>(I->getColor())->getColor();
  }
  
  X3D::TextureCoordinate *TexCoordNode = dynamic_cast<X3D::TextureCoordinate *>(I->getTexCoord());
  if (TexCoordNode != 0)
    texCoordArray = TexCoordNode->getPoint();
  
  coordIndex = I->getCoordIndex();
  colorIndex = I->getColorIndex();
  normalIndex = I->getNormalIndex();
  texCoordIndex = I->getTexCoordIndex();
      
  SFBool normalPerVertex = I->getNormalPerVertex();
  SFBool colorPerVertex = I->getColorPerVertex();
    
  MFInt32::const_iterator itCoordIndex = coordIndex.begin();
  MFInt32::const_iterator itColorIndex;
  MFInt32::const_iterator itNormalIndex;
  MFInt32::const_iterator itTexCoordIndex;
  
  if (colorIndex.empty())
    itColorIndex = coordIndex.begin();
  else
    itColorIndex = colorIndex.begin();  
  
  if (normalIndex.empty())
    itNormalIndex = coordIndex.begin();
  else
    itNormalIndex = normalIndex.begin();  
  
  if (texCoordIndex.empty())
    itTexCoordIndex = coordIndex.begin();
  else
    itTexCoordIndex = texCoordIndex.begin();
    
  MESH::Mesh::MFFace::const_iterator itFace = mesh->getFaces().begin(); 
   
  while (itCoordIndex != coordIndex.end())
  {
    while ((*itCoordIndex != -1) && (itCoordIndex != coordIndex.end()))
    {
      MESH::Mesh::SFVertex *V = mesh->getVertices()[*itCoordIndex];
     
      if (!normalArray.empty())
        V->data().setNormalOfFace(normalArray[*itNormalIndex], *itFace);
      if (!colorArray.empty())
        V->data().setColorOfFace(colorArray[*itColorIndex], *itFace);
      if (!texCoordArray.empty())
        V->data().setTexCoordOfFace(texCoordArray[*itTexCoordIndex], *itFace);  
      
      if (!normalPerVertex)
        (*itFace)->data().setNormal(normalArray[*itNormalIndex]);
      if (!colorPerVertex)
        (*itFace)->data().setColor(colorArray[*itColorIndex]);  
      
      ++itCoordIndex;
      if (colorPerVertex)
        ++itColorIndex;     
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itTexCoordIndex;
    }
    
    ++itCoordIndex;
    ++itColorIndex;
    ++itNormalIndex;
    ++itTexCoordIndex;
    
    ++itFace;
  }
}

void TemplateMeshBuilderCoordinateTraits<MESH::VertexData, MESH::EdgeData, MESH::FaceData, Coordinate>::process(MESH::TemplateVertexSet<MESH::VertexData, MESH::EdgeData, MESH::FaceData> *vertexSet, Coordinate *N)
{
  const MESH::Mesh::MFVertex &mv = vertexSet->getVertices();
  MESH::Mesh::MFVertex::const_iterator itVertex = mv.begin();
  MFVec3f::const_iterator itPoint = N->getPoint().begin();
  
  while (itVertex != mv.end())
  {
    (*itVertex)->data().setPoint(SFPoint3f(*itPoint));
    ++itVertex;
    ++itPoint;
  }
}
  

}
}
