#include "X3D_TemplateMeshBuilderShapeVisitor.h"
#include "X3D_Appearance.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_FillProperties.h"
#include "X3D_LineProperties.h"
#include "X3D_Material.h"
#include "X3D_Shape.h"
#include "X3D_X3DGeometryNode.h"
#include "MESH_X3DGroupingNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TemplateMeshBuilderShapeVisitor::TemplateMeshBuilderShapeVisitor()
: ShapeVisitor()
{
  defineEnterFunction(&TemplateMeshBuilderShapeVisitor::enterShape);
  defineEnterFunction(&TemplateMeshBuilderShapeVisitor::enterX3DAppearanceNode);
  
  defineWalkOnFunction(&TemplateMeshBuilderShapeVisitor::walkOnShape);
  
  defineLeaveFunction(&TemplateMeshBuilderShapeVisitor::leaveShape);
  defineLeaveFunction(&TemplateMeshBuilderShapeVisitor::leaveX3DAppearanceNode);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<TemplateMeshBuilderStateVariables>();
}

TemplateMeshBuilderShapeVisitor::~TemplateMeshBuilderShapeVisitor()
{
}

void TemplateMeshBuilderShapeVisitor::enterShape(Shape *S) const
{
  if (dynamic_cast<IndexedFaceSet *>(S->getGeometry()) == 0)
  {
    if (!stateVariables->hasShape(S))
    {
      Shape *NS = static_cast<Shape *>(stateVariables->clone(S));
      stateVariables->addCoupleShape(S, NS); 
    }
    stateVariables->disableChildrenVisit();      
    
    // Adding child to the father.
    MESH::X3DGroupingNode *G = dynamic_cast<MESH::X3DGroupingNode *>(stateVariables->getTop());
    if (G != 0)
      G->addX3DNodeChild(stateVariables->getShape(S));
  }
  else
  {
    if (!stateVariables->hasShape(S))
      stateVariables->addCoupleShape(S, 0); 
  }  
}

void TemplateMeshBuilderShapeVisitor::enterX3DAppearanceNode(X3DAppearanceNode *N) const
{
  stateVariables->disableChildrenVisit();
  X3DAppearanceNode *NN = static_cast<X3DAppearanceNode *>(stateVariables->clone(N));
  stateVariables->setX3DAppearanceNode(NN);
}

bool TemplateMeshBuilderShapeVisitor::walkOnShape(Shape *, SFAbstractNode child) const
{
  return (stateVariables->getChildrenVisit());
}

void TemplateMeshBuilderShapeVisitor::leaveShape(Shape *) const
{
  stateVariables->enableChildrenVisit();
}

void TemplateMeshBuilderShapeVisitor::leaveX3DAppearanceNode(X3DAppearanceNode *N) const
{
  stateVariables->enableChildrenVisit();
}
  
}
}
