/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DActor.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DACTOR_H
#define X3DACTOR_H

#include "X3DTypes.h"

namespace X3DTK {

class Type;

/// Abstract base class defining an X3DActor.

class X3DTK_API X3DActor 
{
public:  
  /// Constructor.
  X3DActor();
  /// Destructor.
  virtual ~X3DActor() = 0;
  // Deletes the components when the actor is deleted.
  void setAutoDeleteComponents(bool value);
  /// Empties the list of component and delete them if deleteComponents is TRUE
  /// and autoDelete is TRUE.
  virtual void reset() = 0;
  
protected:  
  /// Adds a type.
  virtual void addType(const SFType *type) = 0;
    
  /// If autoDelete is set to TRUE, when the proxy is removed, the components are 
  /// removed too.
  bool autoDelete;
  
  friend class SFType; 
};

}

#endif
